@@ -21,26 +21,26 @@ template<typename T>
 // Use this with counting_iterator to avoid generating a range larger than we
 // can represent.
 template <typename T>
-typename thrust::detail::disable_if<
-  thrust::detail::is_floating_point<T>::value
+typename THRUST_NS_QUALIFIER::detail::disable_if<
+  THRUST_NS_QUALIFIER::detail::is_floating_point<T>::value
 , T
 >::type truncate_to_max_representable(std::size_t n)
 {
-  return thrust::min<std::size_t>(
-    n, static_cast<std::size_t>(thrust::numeric_limits<T>::max())
-  );
+  return static_cast<T>(THRUST_NS_QUALIFIER::min<std::size_t>(
+    n,
+    static_cast<std::size_t>(THRUST_NS_QUALIFIER::numeric_limits<T>::max())));
 }
 
 // TODO: This probably won't work for `half`.
 template <typename T>
-typename thrust::detail::enable_if<
-  thrust::detail::is_floating_point<T>::value
+typename THRUST_NS_QUALIFIER::detail::enable_if<
+  THRUST_NS_QUALIFIER::detail::is_floating_point<T>::value
 , T
 >::type truncate_to_max_representable(std::size_t n)
 {
-  return thrust::min<T>(
-    n, thrust::numeric_limits<T>::max()
-  );
+  return THRUST_NS_QUALIFIER::min<T>(
+    static_cast<T>(n),
+    THRUST_NS_QUALIFIER::numeric_limits<T>::max());
 }
 
 } // end unittest
