@@ -44,8 +44,7 @@
 #  define THRUST_STD_COMPLEX_DEVICE
 #endif
 
-namespace thrust
-{
+THRUST_NAMESPACE_BEGIN
 
 /*
  *  Calls to the standard math library from inside the thrust namespace
@@ -63,9 +62,12 @@ namespace thrust
  *  \{
  */
 
+/*! \cond
+ */
+
 namespace detail
 {
-  
+
 template <typename T, std::size_t Align>
 struct complex_storage;
 
@@ -82,9 +84,9 @@ struct complex_storage;
     || (   (THRUST_HOST_COMPILER == THRUST_HOST_COMPILER_GCC)                 \
         && (THRUST_GCC_VERSION < 40600))
   // C++03 implementation for MSVC and GCC <= 4.5.
-  // 
+  //
   // We have to implement `aligned_type` with specializations for MSVC
-  // and GCC 4.2 and older because they require literals as arguments to 
+  // and GCC 4.2 and older because they require literals as arguments to
   // their alignment attribute.
 
   #if (THRUST_HOST_COMPILER == THRUST_HOST_COMPILER_MSVC)
@@ -115,7 +117,7 @@ struct complex_storage;
   {
     T x; T y;
   };
-  
+
   THRUST_DEFINE_COMPLEX_STORAGE_SPECIALIZATION(1);
   THRUST_DEFINE_COMPLEX_STORAGE_SPECIALIZATION(2);
   THRUST_DEFINE_COMPLEX_STORAGE_SPECIALIZATION(4);
@@ -137,14 +139,17 @@ struct complex_storage;
 
 } // end namespace detail
 
-  /*! \p complex is the Thrust equivalent to <tt>std::complex</tt>. It is
-   *  functionally identical to it, but can also be used in device code which
-   *  <tt>std::complex</tt> currently cannot.
-   *
-   *  \tparam T The type used to hold the real and imaginary parts. Should be
-   *  <tt>float</tt> or <tt>double</tt>. Others types are not supported.
-   *
-   */
+/*! \endcond
+ */
+
+/*! \p complex is the Thrust equivalent to <tt>std::complex</tt>. It is
+ *  functionally identical to it, but can also be used in device code which
+ *  <tt>std::complex</tt> currently cannot.
+ *
+ *  \tparam T The type used to hold the real and imaginary parts. Should be
+ *  <tt>float</tt> or <tt>double</tt>. Others types are not supported.
+ *
+ */
 template <typename T>
 struct complex
 {
@@ -1026,7 +1031,7 @@ template <typename T0, typename T1>
 __host__ __device__
 bool operator!=(const complex<T0>& x, const T1& y);
 
-} // end namespace thrust
+THRUST_NAMESPACE_END
 
 #include <thrust/detail/complex/complex.inl>
 
