@@ -182,7 +182,8 @@ struct bit_not
   template <typename T1>
   __host__ __device__
   constexpr auto operator()(T1&& t1) const
-  noexcept(noexcept(~THRUST_FWD(t1))) -> decltype(~THRUST_FWD(t1))
+  noexcept(noexcept(~THRUST_FWD(t1)))
+  THRUST_TRAILING_RETURN(decltype(~THRUST_FWD(t1)))
   {
     return ~THRUST_FWD(t1);
   }
@@ -212,7 +213,7 @@ struct bit_lshift
   __host__ __device__
   constexpr auto operator()(T1&& t1, T2&& t2) const
   noexcept(noexcept(THRUST_FWD(t1) << THRUST_FWD(t2)))
-      -> decltype(THRUST_FWD(t1) << THRUST_FWD(t2))
+  THRUST_TRAILING_RETURN(decltype(THRUST_FWD(t1) << THRUST_FWD(t2)))
   {
     return THRUST_FWD(t1) << THRUST_FWD(t2);
   }
@@ -276,7 +277,7 @@ struct bit_rshift
   __host__ __device__
   constexpr auto operator()(T1& t1, T2&& t2) const
   noexcept(noexcept(THRUST_FWD(t1) >> THRUST_FWD(t2)))
-  -> decltype(THRUST_FWD(t1) >> THRUST_FWD(t2))
+  THRUST_TRAILING_RETURN(decltype(THRUST_FWD(t1) >> THRUST_FWD(t2)))
   {
     return THRUST_FWD(t1) >> THRUST_FWD(t2);
   }
