@@ -46,7 +46,7 @@ template<typename Operation> struct binary_traits;
  *  Unary Function must define nested \c typedefs. Those \c typedefs are
  *  provided by the base class \p unary_function.
  *
- *  The following code snippet demonstrates how to construct an 
+ *  The following code snippet demonstrates how to construct an
  *  Adaptable Unary Function using \p unary_function.
  *
  *  \code
@@ -86,7 +86,7 @@ struct unary_function
  *  Binary Function must define nested \c typedefs. Those \c typedefs are
  *  provided by the base class \p binary_function.
  *
- *  The following code snippet demonstrates how to construct an 
+ *  The following code snippet demonstrates how to construct an
  *  Adaptable Binary Function using \p binary_function.
  *
  *  \code
@@ -147,7 +147,7 @@ struct binary_function
     template <typename T>                                                      \
     __host__ __device__                                                        \
     constexpr auto operator()(T&& x) const                                     \
-      noexcept(noexcept(impl)) -> decltype(impl)                               \
+      noexcept(noexcept(impl)) THRUST_TRAILING_RETURN(decltype(impl))          \
     {                                                                          \
       return impl;                                                             \
     }                                                                          \
@@ -162,7 +162,7 @@ struct binary_function
     template <typename T1, typename T2>                                        \
     __host__ __device__                                                        \
     constexpr auto operator()(T1&& t1, T2&& t2) const                          \
-      noexcept(noexcept(impl)) -> decltype(impl)                               \
+      noexcept(noexcept(impl)) THRUST_TRAILING_RETURN(decltype(impl))          \
     {                                                                          \
       return impl;                                                             \
     }                                                                          \
@@ -1409,7 +1409,8 @@ struct project1st<void, void>
   template <typename T1, typename T2>
   __host__ __device__
   constexpr auto operator()(T1&& t1, T2&&) const
-    noexcept(noexcept(THRUST_FWD(t1))) -> decltype(THRUST_FWD(t1))
+    noexcept(noexcept(THRUST_FWD(t1)))
+    THRUST_TRAILING_RETURN(decltype(THRUST_FWD(t1)))
   {
     return THRUST_FWD(t1);
   }
@@ -1468,7 +1469,8 @@ struct project2nd<void, void>
   template <typename T1, typename T2>
   __host__ __device__
   constexpr auto operator()(T1&&, T2&& t2) const
-  noexcept(noexcept(THRUST_FWD(t2))) -> decltype(THRUST_FWD(t2))
+  noexcept(noexcept(THRUST_FWD(t2)))
+  THRUST_TRAILING_RETURN(decltype(THRUST_FWD(t2)))
   {
     return THRUST_FWD(t2);
   }
@@ -1495,7 +1497,7 @@ struct project2nd<void, void>
  *  \see not1
  */
 template<typename Predicate>
-struct unary_negate 
+struct unary_negate
     : public thrust::unary_function<typename Predicate::argument_type, bool>
 {
   /*! Constructor takes a \p Predicate object to negate.
@@ -1537,7 +1539,7 @@ template<typename Predicate>
   __host__ __device__
   unary_negate<Predicate> not1(const Predicate &pred);
 
-/*! \p binary_negate is a function object adaptor: it is an Adaptable Binary 
+/*! \p binary_negate is a function object adaptor: it is an Adaptable Binary
  *  Predicate that represents the logical negation of some other Adaptable
  *  Binary Predicate. That is: if \c f is an object of class <tt>binary_negate<AdaptablePredicate></tt>,
  *  then there exists an object \c pred of class \c AdaptableBinaryPredicate
@@ -1564,8 +1566,8 @@ struct binary_negate
   __thrust_exec_check_disable__
   __host__ __device__
   bool operator()(const typename Predicate::first_argument_type& x, const typename Predicate::second_argument_type& y)
-  { 
-      return !pred(x,y); 
+  {
+      return !pred(x,y);
   }
 
   /*! \cond
