@@ -58,14 +58,13 @@ template <
   typename DerivedPolicy
 , typename ForwardIt, typename Size, typename T, typename BinaryOp
 >
-auto async_reduce_n(
+unique_eager_future<remove_cvref_t<T>> async_reduce_n(
   execution_policy<DerivedPolicy>& policy
 , ForwardIt                        first
 , Size                             n
 , T                                init
 , BinaryOp                         op
-) -> unique_eager_future<remove_cvref_t<T>>
-{
+) {
   using U = remove_cvref_t<T>;
 
   auto const device_alloc = get_async_device_allocator(policy);
@@ -89,7 +88,6 @@ auto async_reduce_n(
     , op
     , init
     , nullptr // Null stream, just for sizing.
-    , THRUST_DEBUG_SYNC_FLAG
     )
   , "after reduction sizing"
   );
@@ -171,7 +169,6 @@ auto async_reduce_n(
     , op
     , init
     , fp.future.stream().native_handle()
-    , THRUST_DEBUG_SYNC_FLAG
     )
   , "after reduction launch"
   );
@@ -214,15 +211,14 @@ template <
 , typename ForwardIt, typename Size, typename OutputIt
 , typename T, typename BinaryOp
 >
-auto async_reduce_into_n(
+unique_eager_event async_reduce_into_n(
   execution_policy<DerivedPolicy>& policy
 , ForwardIt                        first
 , Size                             n
 , OutputIt                         output
 , T                                init
 , BinaryOp                         op
-) -> unique_eager_event
-{
+) {
   using U = remove_cvref_t<T>;
 
   auto const device_alloc = get_async_device_allocator(policy);
@@ -242,7 +238,6 @@ auto async_reduce_into_n(
     , op
     , init
     , nullptr // Null stream, just for sizing.
-    , THRUST_DEBUG_SYNC_FLAG
     )
   , "after reduction sizing"
   );
@@ -306,7 +301,6 @@ auto async_reduce_into_n(
     , op
     , init
     , e.stream().native_handle()
-    , THRUST_DEBUG_SYNC_FLAG
     )
   , "after reduction launch"
   );
