@@ -26,15 +26,17 @@
  ******************************************************************************/
 #pragma once
 
-#include <cuda_occupancy.h>
 #include <thrust/detail/config.h>
-#include <thrust/system/cuda/config.h>
-#include <thrust/type_traits/is_contiguous_iterator.h>
 #include <thrust/detail/raw_pointer_cast.h>
+#include <thrust/system/cuda/config.h>
 #include <thrust/system/cuda/detail/util.h>
+#include <thrust/type_traits/is_contiguous_iterator.h>
+
 #include <cub/block/block_load.cuh>
-#include <cub/block/block_store.cuh>
 #include <cub/block/block_scan.cuh>
+#include <cub/block/block_store.cuh>
+
+#include <nv/target>
 
 THRUST_NAMESPACE_BEGIN
 
@@ -350,33 +352,18 @@ namespace core {
     };
 
     template <class Agent>
-    typename get_plan<Agent>::type THRUST_RUNTIME_FUNCTION
-    get_agent_plan(int ptx_version)
+    THRUST_RUNTIME_FUNCTION
+    typename get_plan<Agent>::type get_agent_plan(int ptx_version)
     {
-      // Use one path, with Agent::ptx_plan, for device code where device-side
-      // kernel launches are supported. The other path, with
-      // get_agent_plan_impl::get(version), is for host code and for device
-      // code without device-side kernel launches. NVCC and Feta check for
-      // these situations differently.
-      #ifdef _NVHPC_CUDA
-        #ifdef __THRUST_HAS_CUDART__
-          if (CUB_IS_DEVICE_CODE) {
-            return typename get_plan<Agent>::type(typename Agent::ptx_plan());
-          } else
-        #endif
-        {
-          return get_agent_plan_impl<Agent, sm_list>::get(ptx_version);
-        }
-      #else
-        #if (CUB_PTX_ARCH > 0) && defined(__THRUST_HAS_CUDART__)
-          typedef typename get_plan<Agent>::type Plan;
+      NV_IF_TARGET(
+        NV_IS_DEVICE,
+        (
           THRUST_UNUSED_VAR(ptx_version);
-          // We're on device, use default policy
-          return Plan(typename Agent::ptx_plan());
-        #else
-          return get_agent_plan_impl<Agent, sm_list>::get(ptx_version);
-        #endif
-      #endif
+          using plan_type = typename get_plan<Agent>::type;
+          using ptx_plan  = typename Agent::ptx_plan;
+          return plan_type{ptx_plan{}};
+        ), // NV_IS_HOST:
+        ( return get_agent_plan_impl<Agent, sm_list>::get(ptx_version); ));
     }
 
 // XXX keep this dead-code for now as a gentle reminder
@@ -461,7 +448,7 @@ namespace core {
   /////////////////////////
 
   THRUST_RUNTIME_FUNCTION
-  int get_sm_count()
+  inline int get_sm_count()
   {
     int dev_id;
     cuda_cub::throw_on_error(cudaGetDevice(&dev_id),
@@ -479,8 +466,8 @@ namespace core {
     return i32value;
   }
 
-  size_t THRUST_RUNTIME_FUNCTION
-  get_max_shared_memory_per_block()
+  THRUST_RUNTIME_FUNCTION
+  inline size_t get_max_shared_memory_per_block()
   {
     int dev_id;
     cuda_cub::throw_on_error(cudaGetDevice(&dev_id),
@@ -499,8 +486,8 @@ namespace core {
     return static_cast<size_t>(i32value);
   }
 
-  size_t THRUST_RUNTIME_FUNCTION
-  virtual_shmem_size(size_t shmem_per_block)
+  THRUST_RUNTIME_FUNCTION
+  inline size_t virtual_shmem_size(size_t shmem_per_block)
   {
     size_t max_shmem_per_block = core::get_max_shared_memory_per_block();
     if (shmem_per_block > max_shmem_per_block)
@@ -509,8 +496,8 @@ namespace core {
       return 0;
   }
 
-  size_t THRUST_RUNTIME_FUNCTION
-  vshmem_size(size_t shmem_per_block, size_t num_blocks)
+  THRUST_RUNTIME_FUNCTION
+  inline size_t vshmem_size(size_t shmem_per_block, size_t num_blocks)
   {
     size_t max_shmem_per_block = core::get_max_shared_memory_per_block();
     if (shmem_per_block > max_shmem_per_block)
@@ -606,12 +593,11 @@ namespace core {
   template <class T>
   class cuda_optional
   {
-    cudaError_t status_;
-    T           value_;
+    cudaError_t status_{cudaSuccess};
+    T           value_{};
 
   public:
-    __host__ __device__
-    cuda_optional() : status_(cudaSuccess) {}
+    cuda_optional() = default;
 
     __host__ __device__
     cuda_optional(T v, cudaError_t status = cudaSuccess) : status_(status), value_(v) {}
@@ -628,16 +614,16 @@ namespace core {
     __host__ __device__ operator T const &() const { return value_; }
   };
 
-  cuda_optional<int> THRUST_RUNTIME_FUNCTION
-  get_ptx_version()
+  THRUST_RUNTIME_FUNCTION
+  inline cuda_optional<int> get_ptx_version()
   {
     int ptx_version = 0;
     cudaError_t status = cub::PtxVersion(ptx_version);
     return cuda_optional<int>(ptx_version, status);
   }
 
-  cudaError_t THRUST_RUNTIME_FUNCTION
-  sync_stream(cudaStream_t stream)
+  THRUST_RUNTIME_FUNCTION
+  inline cudaError_t sync_stream(cudaStream_t stream)
   {
     return cub::SyncStream(stream);
   }
