@@ -26,11 +26,12 @@
  ******************************************************************************/
 #pragma once
 
+#include <thrust/detail/config.h>
+
 #include <thrust/detail/seq.h>
 #include <thrust/system/cuda/detail/par.h>
 
-namespace thrust
-{
+THRUST_NAMESPACE_BEGIN
 namespace cuda_cub {
 
 template <int PAR>
@@ -81,11 +82,5 @@ cvt_to_seq(Policy& policy)
   return cvt_to_seq_impl<Policy>::doit(policy);
 }
 
-#if __THRUST_HAS_CUDART__
-#define THRUST_CUDART_DISPATCH par
-#else
-#define THRUST_CUDART_DISPATCH seq
-#endif
-
 } // namespace cuda_
-} // end namespace thrust
+THRUST_NAMESPACE_END
