@@ -92,11 +92,6 @@ swap_ranges(execution_policy<Derived> &policy,
                                                ItemsIt2>(first1, first2),
                          num_items);
 
-  cuda_cub::throw_on_error(
-    cuda_cub::synchronize_optional(policy)
-  , "swap_ranges: failed to synchronize"
-  );
-
   return first2 + num_items;
 }
 
