@@ -27,8 +27,7 @@
 #include <thrust/detail/function.h>
 #include <thrust/system/detail/sequential/execution_policy.h>
 
-namespace thrust
-{
+THRUST_NAMESPACE_BEGIN
 namespace detail
 {
 
@@ -96,7 +95,7 @@ __host__ __device__
     if(wrapped_pred(*next))
     {
       // Fully qualify name to disambiguate overloads found via ADL.
-      ::thrust::system::detail::sequential::iter_swap(first, next);
+      THRUST_NS_QUALIFIER::system::detail::sequential::iter_swap(first, next);
       ++first;
     }
   }
@@ -145,7 +144,7 @@ __host__ __device__
     if(wrapped_pred(*stencil_first))
     {
       // Fully qualify name to disambiguate overloads found via ADL.
-      ::thrust::system::detail::sequential::iter_swap(first, next);
+      THRUST_NS_QUALIFIER::system::detail::sequential::iter_swap(first, next);
       ++first;
     }
 
@@ -337,5 +336,5 @@ __host__ __device__
 } // end namespace sequential
 } // end namespace detail
 } // end namespace system
-} // end namespace thrust
+THRUST_NAMESPACE_END
 
